@@ -112,6 +112,7 @@ static GstMessageQuarks message_quarks[] = {
   {GST_MESSAGE_REQUEST_STATE, "request-state", 0},
   {GST_MESSAGE_STEP_START, "step-start", 0},
   {GST_MESSAGE_QOS, "qos", 0},
+  {GST_MESSAGE_PROGRESS, "progress", 0},
   {0, NULL, 0}
 };
 
@@ -2079,3 +2080,69 @@ gst_message_parse_qos_stats (GstMessage * message, GstFormat * format,
       GST_QUARK (PROCESSED), G_TYPE_UINT64, processed,
       GST_QUARK (DROPPED), G_TYPE_UINT64, dropped, NULL);
 }
+
+/**
+ * gst_message_new_progress:
+ * @src: The object originating the message.
+ * @type: a #GstProgressType
+ * @code: a progress code
+ * @text: free, user visible text describing the progress
+ *
+ * Progress messages are posted by elements when they use an asynchronous task
+ * to perform actions triggered by a state change.
+ *
+ * @code contains a well defined string describing the action.
+ * @test should contain a user visible string detailing the current action.
+ *
+ * Returns: (transfer full): The new qos message.
+ *
+ * Since: 0.10.33
+ */
+GstMessage *
+gst_message_new_progress (GstObject * src, GstProgressType type,
+    const gchar * code, const gchar * text)
+{
+  GstMessage *message;
+  GstStructure *structure;
+  gint percent = 100, timeout = -1;
+
+  g_return_val_if_fail (code != NULL, NULL);
+  g_return_val_if_fail (text != NULL, NULL);
+
+  if (type == GST_PROGRESS_TYPE_START || type == GST_PROGRESS_TYPE_CONTINUE)
+    percent = 0;
+
+  structure = gst_structure_id_new (GST_QUARK (MESSAGE_PROGRESS),
+      GST_QUARK (TYPE), GST_TYPE_PROGRESS_TYPE, type,
+      GST_QUARK (CODE), G_TYPE_STRING, code,
+      GST_QUARK (TEXT), G_TYPE_STRING, text,
+      GST_QUARK (PERCENT), G_TYPE_INT, percent,
+      GST_QUARK (TIMEOUT), G_TYPE_INT, timeout, NULL);
+  message = gst_message_new_custom (GST_MESSAGE_PROGRESS, src, structure);
+
+  return message;
+}
+
+/**
+ * gst_message_parse_progress:
+ * @message: A valid #GstMessage of type GST_MESSAGE_PROGRESS.
+ * @type: (out) (allow-none): location for the type
+ * @code: (out) (allow-none) (transfer full): location for the code
+ * @text: (out) (allow-none) (transfer full): location for the text
+ *
+ * Parses the progress @type, @code and @text.
+ *
+ * Since: 0.10.33
+ */
+void
+gst_message_parse_progress (GstMessage * message, GstProgressType * type,
+    gchar ** code, gchar ** text)
+{
+  g_return_if_fail (GST_IS_MESSAGE (message));
+  g_return_if_fail (GST_MESSAGE_TYPE (message) == GST_MESSAGE_PROGRESS);
+
+  gst_structure_id_get (message->structure,
+      GST_QUARK (TYPE), GST_TYPE_PROGRESS_TYPE, type,
+      GST_QUARK (CODE), G_TYPE_STRING, code,
+      GST_QUARK (TEXT), G_TYPE_STRING, text, NULL);
+}
