@@ -25,6 +25,13 @@
  * @see_also: #GstFileSrc
  *
  * Write incoming data to a file in the local file system.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch v4l2src num-buffers=1 ! jpegenc ! filesink location=capture1.jpeg
+ * ]| Capture one frame from a v4l2 camera and save as jpeg image.
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -237,7 +244,6 @@ gst_file_sink_class_init (GstFileSinkClass * klass)
           "Append to an already existing file", DEFAULT_APPEND,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gstbasesink_class->get_times = NULL;
   gstbasesink_class->start = GST_DEBUG_FUNCPTR (gst_file_sink_start);
   gstbasesink_class->stop = GST_DEBUG_FUNCPTR (gst_file_sink_stop);
   gstbasesink_class->render = GST_DEBUG_FUNCPTR (gst_file_sink_render);
@@ -296,7 +302,9 @@ gst_file_sink_set_location (GstFileSink * sink, const gchar * location)
     /* we store the filename as we received it from the application. On Windows
      * this should be in UTF8 */
     sink->filename = g_strdup (location);
-    sink->uri = gst_uri_construct ("file", sink->filename);
+    sink->uri = gst_filename_to_uri (location, NULL);
+    GST_INFO ("filename : %s", sink->filename);
+    GST_INFO ("uri      : %s", sink->uri);
   } else {
     sink->filename = NULL;
     sink->uri = NULL;
