@@ -112,6 +112,7 @@
 #include <gst/gst.h>
 #include <stdio.h>
 #include "gstmultiqueue.h"
+#include <gst/glib-compat-private.h>
 
 /**
  * GstSingleQueue:
@@ -1240,7 +1241,7 @@ gst_multi_queue_chain (GstPad * pad, GstBuffer * buffer)
     goto was_eos;
 
   /* Get a unique incrementing id */
-  curid = g_atomic_int_exchange_and_add ((gint *) & mq->counter, 1);
+  curid = G_ATOMIC_INT_ADD ((gint *) & mq->counter, 1);
 
   GST_LOG_OBJECT (mq, "SingleQueue %d : about to enqueue buffer %p with id %d",
       sq->id, buffer, curid);
@@ -1346,7 +1347,7 @@ gst_multi_queue_sink_event (GstPad * pad, GstEvent * event)
     goto was_eos;
 
   /* Get an unique incrementing id. */
-  curid = g_atomic_int_exchange_and_add ((gint *) & mq->counter, 1);
+  curid = G_ATOMIC_INT_ADD ((gint *) & mq->counter, 1);
 
   item = gst_multi_queue_event_item_new ((GstMiniObject *) event, curid);
 
