@@ -252,6 +252,7 @@ my_buffer_pool_drain_one (MyBufferPool * self)
   return buf;
 }
 
+GType my_recycle_buffer_get_type (void);
 G_DEFINE_TYPE (MyRecycleBuffer, my_recycle_buffer, GST_TYPE_BUFFER);
 
 static void my_recycle_buffer_finalize (GstMiniObject * mini_object);
@@ -365,6 +366,7 @@ enum
   PROP_BUFFER = 1
 };
 
+GType my_foo_get_type (void);
 G_DEFINE_TYPE (MyFoo, my_foo, G_TYPE_OBJECT);
 
 static void
@@ -445,6 +447,23 @@ GST_START_TEST (test_value_collection)
 GST_END_TEST;
 
 
+GST_START_TEST (test_dup_null_mini_object)
+{
+  GValue value = { 0, };
+  GstMiniObject *mo;
+
+  g_value_init (&value, GST_TYPE_BUFFER);
+
+  gst_value_set_mini_object (&value, NULL);
+
+  mo = gst_value_dup_mini_object (&value);
+  g_assert (mo == NULL);
+
+  g_value_unset (&value);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_mini_object_suite (void)
 {
@@ -462,6 +481,7 @@ gst_mini_object_suite (void)
   tcase_add_test (tc_chain, test_unref_threaded);
   tcase_add_test (tc_chain, test_recycle_threaded);
   tcase_add_test (tc_chain, test_value_collection);
+  tcase_add_test (tc_chain, test_dup_null_mini_object);
   return s;
 }
 
